/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import java.util.Optional;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.springframework.stereotype.Component;

import de.ozgcloud.operator.keycloak.KeycloakGenericRemoteService;
import de.ozgcloud.operator.keycloak.user.OzgCloudKeycloakUserSpec.KeycloakUserSpecClientRole;
import de.ozgcloud.operator.keycloak.user.OzgCloudKeycloakUserSpec.KeycloakUserSpecUserGroup;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
class KeycloakUserPreconditionService {

	private final KeycloakGenericRemoteService keycloakGenericRemoteService;

	public Optional<String> getPreconditionErrors(OzgCloudKeycloakUser user) {
		var namespace = user.getMetadata().getNamespace();

		if (!realmExists(namespace)) {
			return Optional.of(String.format("Realm %s does not yet exist", namespace));
		}

		var clientErorr = clientsExists(user, namespace);
		if (clientErorr.isPresent()) {
			return clientErorr;
		}

		var groupError = groupsExists(user, namespace);
		if (groupError.isPresent()) {
			return groupError;
		}

		if (!userEmailExistsAndIsValidEmail(user)) {
			return Optional.of(String.format("User Email does not yet exist"));
		}

		return Optional.empty();
	}

	boolean realmExists(String realmName) {
		return keycloakGenericRemoteService.realmExists(realmName);
	}

	Optional<String> clientsExists(OzgCloudKeycloakUser user, String realm) {
		return user.getSpec().getKeycloakUser().getClientRoles().stream()
				.map(KeycloakUserSpecClientRole::getClientId)
				.map(clientId -> keycloakGenericRemoteService.getByClientId(clientId, realm))
				.filter(Optional::isEmpty)
				.map(clientId -> String.format("Client %s does not yet exist", clientId))
				.findAny();
	}

	Optional<String> groupsExists(OzgCloudKeycloakUser user, String realm) {
		return user.getSpec().getKeycloakUser().getGroups().stream()
				.map(KeycloakUserSpecUserGroup::getName)
				.filter(groupName -> !keycloakGenericRemoteService.groupExists(groupName, realm))
				.map(groupName -> String.format("Group %s for realm %s does not exist yet", groupName, realm))
				.findAny();
	}

	boolean userEmailExistsAndIsValidEmail(OzgCloudKeycloakUser user) {
		String email = user.getSpec().getKeycloakUser().getEmail();
		return !Optional.ofNullable(email).orElse("").isEmpty() && isValidEmail(email);
	}

	boolean isValidEmail(String email) {
		String EMAIL_REGEX = "^[A-Za-z0-9+_.-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,6}$";
		Pattern EMAIL_PATTERN = Pattern.compile(EMAIL_REGEX);

		Matcher matcher = EMAIL_PATTERN.matcher(email);
		return matcher.matches();
	}
}
