/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.realm;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.keycloak.representations.idm.RealmRepresentation;
import org.mapstruct.factory.Mappers;
import org.mockito.Spy;

import de.ozgcloud.operator.keycloak.realm.OzgCloudKeycloakRealmSpec.KeycloakRealmSMTPServer;

class KeycloakRealmMapperTest {

	@Spy
	private final KeycloakRealmMapper mapper = Mappers.getMapper(KeycloakRealmMapper.class);

	private RealmRepresentation mapRealm() {
		return mapper.map(OzgCloudKeycloakRealmSpecTestFactory.create());

	}

	@Test
	void shouldMapDisplayName() {
		var mapped = mapRealm();

		assertThat(mapped.getDisplayName()).isEqualTo(OzgCloudKeycloakRealmSpecTestFactory.DISPLAY_NAME);
	}

	@Test
	void shouldBeEnabled() {
		var mapped = mapRealm();

		assertThat(mapped.isEnabled()).isTrue();
	}

	@Test
	void shouldBeResetPasswordAllowed() {
		var mapped = mapRealm();

		assertThat(mapped.isResetPasswordAllowed()).isTrue();
	}

	@Test
	void shouldBeSupportedLocaleDe() {
		var mapped = mapRealm();

		assertThat(mapped.getSupportedLocales()).containsExactly("de");
	}

	@Test
	void shouldBeDefaultLocaleDe() {
		var mapped = mapRealm();

		assertThat(mapped.getDefaultLocale()).isEqualTo("de");
	}

	@Test
	void shouldBeInternationalizationEnabled() {
		var mapped = mapRealm();

		assertThat(mapped.isInternationalizationEnabled()).isTrue();
	}

	@Test
	void checkPasswordPolicy() {
		var mapped = mapRealm();

		assertThat(mapped.getPasswordPolicy()).isEqualTo("upperCase(1) and lowerCase(1) and length(8) and notUsername");
	}

	@Test
	void shouldSetActionTokenGeneratedByUserLifespan() {
		var mapped = mapRealm();

		assertThat(mapped.getActionTokenGeneratedByUserLifespan()).isEqualTo(900);
	}

	@Test
	void shouldMapSmtpServer() {
		var mapped = mapRealm();

		assertThat(mapped.getSmtpServer()).hasSize(8);
	}

	@Test
	void shouldContainSmtpServerKeysValues() {
		var mapped = mapRealm();

		assertThat(mapped.getSmtpServer())
				.containsEntry(KeycloakRealmSMTPServer.USER_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_USER)
				.containsEntry(KeycloakRealmSMTPServer.PASSWORD_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_PASSWORD)
				.containsEntry(KeycloakRealmSMTPServer.HOST_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_HOST)
				.containsEntry(KeycloakRealmSMTPServer.PORT_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_PORT)
				.containsEntry(KeycloakRealmSMTPServer.STARTTLS_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_STARTTLS)
				.containsEntry(KeycloakRealmSMTPServer.AUTH_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_AUTH)
				.containsEntry(KeycloakRealmSMTPServer.FROM_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_FROM)
				.containsEntry(KeycloakRealmSMTPServer.FROM_DISPLAY_NAME_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_FROM_DISPLAY_NAME);
	}

	@Test
	void shouldMapWhenSmtpServerIsNull() {
		var mapped = mapper.map(OzgCloudKeycloakRealmSpecTestFactory.createBuilder().smtpServer(null).build());

		assertThat(mapped.getSmtpServer()).isEmpty();
	}

	@Test
	void shouldMapRealmRoles() {
		var mapped = mapRealm();

		assertThat(mapped.getRoles().getRealm()).hasSize(2);
	}

	@Test
	void shouldContainRealmRolesValues() {
		var mapped = mapRealm();
		var mappedRealmRoles = mapped.getRoles();

		assertThat(mappedRealmRoles.getRealm().get(0).getName()).isEqualTo(OzgCloudKeycloakRealmSpecTestFactory.ROLE_NAME_1);
		assertThat(mappedRealmRoles.getRealm().get(1).getName()).isEqualTo(OzgCloudKeycloakRealmSpecTestFactory.ROLE_NAME_2);
	}

	@Nested
	class TestUpdate {

		@Test
		void shouldMapDisplayName() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(), OzgCloudKeycloakRealmSpecTestFactory.create());

			assertThat(mapped.getDisplayName()).isEqualTo(OzgCloudKeycloakRealmSpecTestFactory.DISPLAY_NAME);
		}

		@Test
		void shouldBeEnabled() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(), OzgCloudKeycloakRealmSpecTestFactory.create());

			assertThat(mapped.isEnabled()).isTrue();
		}

		@Test
		void shouldBeResetPasswordAllowed() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(), OzgCloudKeycloakRealmSpecTestFactory.create());

			assertThat(mapped.isResetPasswordAllowed()).isTrue();
		}

		@Test
		void shouldBeSupportedLocaleDe() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(), OzgCloudKeycloakRealmSpecTestFactory.create());

			assertThat(mapped.getSupportedLocales()).containsExactly("de");
		}

		@Test
		void shouldBeDefaultLocaleDe() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(), OzgCloudKeycloakRealmSpecTestFactory.create());

			assertThat(mapped.getDefaultLocale()).isEqualTo("de");
		}

		@Test
		void shouldBeInternationalizationEnabled() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(), OzgCloudKeycloakRealmSpecTestFactory.create());

			assertThat(mapped.isInternationalizationEnabled()).isTrue();
		}

		@Test
		void checkPasswordPolicy() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(), OzgCloudKeycloakRealmSpecTestFactory.create());

			assertThat(mapped.getPasswordPolicy()).isEqualTo("upperCase(1) and lowerCase(1) and length(8) and notUsername");
		}

		@Test
		void shouldSetActionTokenGeneratedByUserLifespan() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(), OzgCloudKeycloakRealmSpecTestFactory.create());

			assertThat(mapped.getActionTokenGeneratedByUserLifespan()).isEqualTo(900);
		}

		@Test
		void shouldMapSmtpServer() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(), OzgCloudKeycloakRealmSpecTestFactory.create());

			assertThat(mapped.getSmtpServer()).hasSize(8);
		}

		@Test
		void shouldContainSmtpServerKeysValues() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(), OzgCloudKeycloakRealmSpecTestFactory.create());

			assertThat(mapped.getSmtpServer())
					.containsEntry(KeycloakRealmSMTPServer.USER_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_USER)
					.containsEntry(KeycloakRealmSMTPServer.PASSWORD_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_PASSWORD)
					.containsEntry(KeycloakRealmSMTPServer.HOST_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_HOST)
					.containsEntry(KeycloakRealmSMTPServer.PORT_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_PORT)
					.containsEntry(KeycloakRealmSMTPServer.STARTTLS_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_STARTTLS)
					.containsEntry(KeycloakRealmSMTPServer.AUTH_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_AUTH)
					.containsEntry(KeycloakRealmSMTPServer.FROM_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_FROM)
					.containsEntry(KeycloakRealmSMTPServer.FROM_DISPLAY_NAME_FIELD, KeycloakRealmSmtpServerTestFactory.SMTP_SERVER_FROM_DISPLAY_NAME);
		}

		@Test
		void shouldMapWhenSmtpServerIsNull() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(),
					OzgCloudKeycloakRealmSpecTestFactory.createBuilder().smtpServer(null).build());

			assertThat(mapped.getSmtpServer()).isEmpty();
		}

		@Test
		void shouldMapRealmRoles() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(), OzgCloudKeycloakRealmSpecTestFactory.create());

			assertThat(mapped.getRoles().getRealm()).hasSize(2);
		}

		@Test
		void shouldContainRealmRolesValues() {
			var mapped = mapper.update(RealmRepresentationTestFactory.create(), OzgCloudKeycloakRealmSpecTestFactory.create());
			var mappedRealmRoles = mapped.getRoles();

			assertThat(mappedRealmRoles.getRealm().get(0).getName()).isEqualTo(OzgCloudKeycloakRealmSpecTestFactory.ROLE_NAME_1);
			assertThat(mappedRealmRoles.getRealm().get(1).getName()).isEqualTo(OzgCloudKeycloakRealmSpecTestFactory.ROLE_NAME_2);
		}

	}
}
